@@ -55,7 +55,9 @@
 #endif /* ENETUNREACH */
 
 #ifdef SSL_ENABLE
+#include "tls-aux.h"		/* compatibility and helper functions */
 #include <openssl/ssl.h>	/* for OPENSSL_NO_SSL2 and ..._SSL3 checks */
+#include <openssl/opensslv.h>	/* for version queries */
 #endif
 
 /* prototypes for internal functions */
@@ -144,11 +146,11 @@ static void printcopyright(FILE *fp) {
 		   "                   Robert M. Funk, Graham Wilson\n"
 		   "Copyright (C) 2005 - 2012 Sunil Shetye\n"
 		   "Copyright (C) 2005 - %d Matthias Andree\n"
-		   ), 2020);
+		   ), 2021);
 	fprintf(fp, GT_("Fetchmail comes with ABSOLUTELY NO WARRANTY. This is free software, and you\n"
 		   "are welcome to redistribute it under certain conditions. For details,\n"
 		   "please see the file COPYING in the source or documentation directory.\n"));
-#ifdef SSL_ENABLE
+#if defined(SSL_ENABLE) && !defined(USING_WOLFSSL)
 	/* Do not translate this */
 	fprintf(fp, "This product includes software developed by the OpenSSL Project\nfor use in the OpenSSL Toolkit. (http://www.openssl.org/)\n");
 #endif
@@ -267,8 +269,12 @@ int main(int argc, char **argv)
 #endif /* ODMR_ENABLE */
 #ifndef SSL_ENABLE
 	"-SSL"
+#else
+#ifdef USING_WOLFSSL
+	"+WOLFSSL"
 #else
 	"+SSL-SSLv2"
+#endif
 #if (HAVE_DECL_SSLV3_CLIENT_METHOD + 0 == 0) || defined(OPENSSL_NO_SSL3)
 	"-SSLv3"
 #endif
@@ -304,12 +310,22 @@ int main(int argc, char **argv)
 	printf(GT_("This is fetchmail release %s"), VERSION);
 	fputs(features, stdout);
 #ifdef SSL_ENABLE
-#if !HAVE_DECL_TLS1_3_VERSION || defined(OPENSSL_NO_TLS1_3)
+	printf(GT_("Compiled with SSL library %#lx \"%s\"\n"
+		   "Run-time uses SSL library %#lx \"%s\"\n"),
+			OPENSSL_VERSION_NUMBER, OPENSSL_VERSION_TEXT,
+			OpenSSL_version_num(), OpenSSL_version(OPENSSL_VERSION));
+	printf(GT_("OpenSSL: %s\nEngines: %s\n"),
+			OpenSSL_version(OPENSSL_DIR),
+			OpenSSL_version(OPENSSL_ENGINES_DIR));
+# if !HAVE_DECL_TLS1_3_VERSION || defined(OPENSSL_NO_TLS1_3)
 	printf(GT_("WARNING: Your SSL/TLS library does not support TLS v1.3.\n"));
-#endif
-#ifdef LIBRESSL_VERSION_NUMBER
-	printf(GT_("WARNING: Compiled against LibreSSL, which is not a supported configuration.\n"));
-#endif
+# endif
+# if defined(LIBRESSL_VERSION_NUMBER) && !defined(__OpenBSD__)
+	/* OpenBSD ships LibreSSL as part of the base system, so is exempt
+	 * because it can pull the GPL v2 clause 3 exception */
+	printf(GT_("ERROR: Compiled against LibreSSL, which is a copyright violation for lack of GPL clause 2b exception. See COPYING. Aborting.\n"));
+	exit(PS_UNDEFINED);
+# endif
 #else
 	printf(GT_("WARNING: Compiled without SSL/TLS.\n"));
 #endif
@@ -329,9 +345,10 @@ int main(int argc, char **argv)
 	if (system("uname -a")) { /* NOOP to quench GCC complaint */ }
     }
 
-    /* avoid parsing the config file if all we're doing is killing a daemon */
-    if (!quitonly)
-	implicitmode = load_params(argc, argv, optind);
+    /* We used to avoid parsing the config file if all we're doing is killing 
+     * a daemon, under if (!quitonly) but since the pidfile can be configured 
+     * in the rcfile, this is no longer viable. */
+    implicitmode = load_params(argc, argv, optind);
 
     if (run.logfile) {
 	/* nodetach -> turn off logfile option */
@@ -340,13 +357,13 @@ int main(int argc, char **argv)
 	    xfree(run.logfile);
 	}
 
-#if 0
+# if 0
 	/* not in daemon mode -> turn off logfile option */
 	if (0 == run.poll_interval) {
 	    if (outlevel >= O_NORMAL) { fprintf(stderr, GT_("Not running in daemon mode, ignoring logfile option.\n")); }
 	    xfree(run.logfile);
 	}
-#endif
+# endif
 
 	/* log file not writable -> turn off logfile option */
 	if (run.logfile && 0 != access(run.logfile, F_OK)) {
@@ -755,7 +772,7 @@ int main(int argc, char **argv)
 	/* Boldly assume that we also have res_init() if we have
 	 * res_search(), and call res_init() to re-read the resolv.conf
 	 * file, so that we can pick up changes to that file that are
-	 * written by dhpccd, dhclient, pppd, openvpn and similar. */
+	 * written by dhcpcd, dhclient, pppd, openvpn and similar. */
 
 	/* NOTE: This assumes that /etc/resolv.conf is written
 	 * atomically (i. e. a temporary file is written, flushed and
@@ -986,7 +1003,7 @@ static void optmerge(struct query *h2, struct query *h1, int force)
     list_merge(&h2->antispam, &h1->antispam, force);
 
 #define   FLAG_MERGE(fld) do { if (force ? !!h1->fld : !h2->fld) h2->fld = h1->fld; } while (0)
-#define STRING_MERGE(fld) do { if (force ? !!h1->fld : !h2->fld) { if (h2->fld) free((void *)h2->fld), h2->fld = 0; if (h1->fld) h2->fld = xstrdup(h1->fld); } } while (0)
+#define STRING_MERGE(fld) do { if (force ? !!h1->fld : !h2->fld) { if (h2->fld) free((void *)h2->fld), h2->fld = 0; if (h1->fld) { if (h1->fld != STRING_DISABLED) h2->fld = xstrdup(h1->fld); else h2->fld = STRING_DISABLED; } } } while (0)
     STRING_MERGE(server.via);
     FLAG_MERGE(server.protocol);
     STRING_MERGE(server.service);
@@ -1402,6 +1419,16 @@ static int load_params(int argc, char **argv, int optind)
 				   ctl->server.pollname);
 		    exit(PS_SYNTAX);
 		}
+		switch (ctl->server.protocol) {
+			case P_POP3: case P_APOP:
+				if (port == 995 && !ctl->use_ssl) report(stderr, GT_("WARNING: %s configuration invalid, you normally need --ssl for port 995/service pop3s.\n"), ctl->server.pollname);
+				if (port == 110 &&  ctl->use_ssl) report(stderr, GT_("WARNING: %s configuration invalid, you normally need port 995/service pop3s for --ssl.\n"), ctl->server.pollname);
+				break;
+			case P_IMAP:
+				if (port == 993 && !ctl->use_ssl) report(stderr, GT_("WARNING: %s configuration invalid, you normally need --ssl for port 993/service imaps.\n"), ctl->server.pollname);
+				if (port == 143 &&  ctl->use_ssl) report(stderr, GT_("WARNING: %s configuration invalid, you normally need port 993/service imaps for --ssl.\n"), ctl->server.pollname);
+				break;
+		}
 	    }
 	    if (ctl->listener == LMTP_MODE)
 	    {
@@ -1411,9 +1438,10 @@ static int load_params(int argc, char **argv, int optind)
 		{
 		    char	*cp;
 
-		    if (!(cp = strrchr(idp->id, '/'))
-			|| (0 == strcmp(cp + 1, SMTP_PORT))
-			|| servport(cp + 1) == SMTP_PORT_NUM)
+		    if ((idp->id[0] != '/') /* do not port-check UNIX paths */ && 
+				    (!(cp = strrchr(idp->id, '/'))
+				     || (0 == strcmp(cp + 1, SMTP_PORT))
+				     || servport(cp + 1) == SMTP_PORT_NUM))
 		    {
 			(void) fprintf(stderr,
 				       GT_("%s configuration invalid, LMTP can't use default SMTP port\n"),
@@ -1768,8 +1796,10 @@ static void dump_params (struct runctl *runp,
 	} else {
 	    printf(GT_("  SSL server certificate checking disabled.\n"));
 	}
+	printf(GT_("  SSL default trusted certificate file: %s\n"), get_default_cert_file());
 	if (ctl->sslcertfile != NULL)
 		printf(GT_("  SSL trusted certificate file: %s\n"), ctl->sslcertfile);
+	printf(GT_("  SSL default trusted certificate directory: %s\n"), get_default_cert_path());
 	if (ctl->sslcertpath != NULL)
 		printf(GT_("  SSL trusted certificate directory: %s\n"), ctl->sslcertpath);
 	if (ctl->sslcommonname != NULL)
