@@ -26,15 +26,34 @@ import getopt
 import tempfile
 import ssl
 import subprocess
-from tkinter import *
-from tkinter.dialog import *
+try:
+    from tkinter import *
+    from tkinter.dialog import *
+    have_tk=True
+except:
+    have_tk=False
+    # define a dummy class to inherit from
+    class Frame: pass
 
-VERSION = "1.63.1"
+VERSION = "1.63.5"
 
 MIN_PY = (2, 7, 13)
 if sys.version_info < MIN_PY:
     sys.exit("fetchmailconf: Python %s.%s.%s or later is required.\n" % MIN_PY)
 
+#
+# Display usage information and pass status argument through to sys.exit()
+#
+def usage(status):
+    print("""
+Usage: fetchmailconf {[-d] [-f fetchmailrc]|-h|--help|-V|--version}
+           -d      - dump configuration (for debugging)
+           -f fmrc - read alternate fetchmailrc file
+--help,    -h      - print this help text and quit
+--version, -V      - print fetchmailconf version and quit
+""")
+    sys.exit(status)
+
 #
 # Define the data structures the GUIs will be tossing around
 #
@@ -294,7 +313,8 @@ class User(object):
         self.sslcert = None	# SSL certificate filename
         self.sslproto = None	# Force SSL?
         self.sslcertck = True	# Enable strict SSL cert checking
-        self.sslcertpath = None	# Path to trusted certificates
+        self.sslcertfile = None	# Path to bundled file with trusted certificates
+        self.sslcertpath = None	# Path to hashed directory trusted certificates
         self.sslcommonname = None	# SSL CommonName to expect
         self.sslfingerprint = None	# SSL key fingerprint to check
         self.properties = None	# Extension properties
@@ -334,6 +354,7 @@ class User(object):
             ('sslkey',	    'String'),
             ('sslcert',     'String'),
             ('sslcertck',   'Boolean'),
+            ('sslcertfile', 'String'),
             ('sslcertpath', 'String'),
             ('sslcommonname', 'String'),
             ('sslfingerprint', 'String'),
@@ -408,6 +429,8 @@ class User(object):
             res = res + " sslproto " + repr(self.sslproto)
         if self.sslcertck is not None and self.sslcertck != UserDefaults.sslcertck:
             res = res +  flag2str(self.sslcertck, 'sslcertck')
+        if self.sslcertfile and self.sslcertfile != UserDefaults.sslcertfile:
+            res = res + " sslcertfile " + repr(self.sslcertfile)
         if self.sslcertpath and self.sslcertpath != UserDefaults.sslcertpath:
             res = res + " sslcertpath " + repr(self.sslcertpath)
         if self.sslcommonname and self.sslcommonname != UserDefaults.sslcommonname:
@@ -491,7 +514,7 @@ def flag2str(value, string):
     res = ""
     if value != None:
         res = res + (" ")
-        if value == FALSE:
+        if not value:
             res = res + ("no ")
         res = res + string
     return res
@@ -1772,7 +1795,9 @@ class UserEdit(Frame, MyWidget):
                          self.sslcert, '14').pack(side=TOP, fill=X)
             Checkbutton(sslwin, text="Check server SSL certificate?",
                         variable=self.sslcertck).pack(side=TOP, fill=X)
-            LabeledEntry(sslwin, 'SSL trusted certificate directory:',
+            LabeledEntry(sslwin, 'SSL trust bundle:',
+                         self.sslcertfile, '14').pack(side=TOP, fill=X)
+            LabeledEntry(sslwin, 'SSL trust directory:',
                          self.sslcertpath, '14').pack(side=TOP, fill=X)
             LabeledEntry(sslwin, 'SSL CommonName:',
                          self.sslcommonname, '14').pack(side=TOP, fill=X)
@@ -2173,35 +2198,36 @@ gUSiYASJpMEHhilJTEnhAlGoQqYAZQ1AiqEMZ0jDGtqQImhwwA13yMMevoQAGvGhEAWHGMOAAAA7
 #
 
     # Process options
-    options, arguments = getopt.getopt(sys.argv[1:], "df:hV", ["help", "version"])
+    try:
+        options, arguments = getopt.getopt(sys.argv[1:], "df:hV", ["help", "version"])
+    except getopt.GetoptError as e:
+        usage(str(e))
+    except Exception as e:
+        raise e
     dump = rcfile = None
     for (switch, val) in options:
         if switch == '-d':
-            dump = TRUE
+            dump = True
         elif switch == '-f':
             rcfile = val
         elif switch == '-h' or switch == '--help':
-            print("""
-Usage: fetchmailconf {[-d] [-f fetchmailrc]|-h|--help|-V|--version}
-           -d      - dump configuration (for debugging)
-           -f fmrc - read alternate fetchmailrc file
---help,    -h      - print this help text and quit
---version, -V      - print fetchmailconf version and quit
-""")
-            sys.exit(0)
+            usage(0)
         elif switch == '-V' or switch == '--version':
             print("fetchmailconf %s" % VERSION)
+            print("Running on python", sys.version)
             print("""
 Copyright (C) 1997 - 2003 Eric S. Raymond
-Copyright (C) 2005 - 2020 Matthias Andree
+Copyright (C) 2005 - 2021 Matthias Andree
 fetchmailconf comes with ABSOLUTELY NO WARRANTY.  This is free software, you are
 welcome to redistribute it under certain conditions.  Please see the file
 COPYING in the source or documentation directory for details.""")
             sys.exit(0)
 
+    if arguments:
+        usage("Extra arguments: '" + "' '".join(arguments) + "'")
+
     if "DISPLAY" not in os.environ:
-        print("fetchmailconf must be run under X")
-        sys.exit(1)
+        sys.exit("fetchmailconf must be run under X")
 
     # Get client host's FQDN
     hostname = socket.gethostname()
@@ -2211,12 +2237,7 @@ COPYING in the source or documentation directory for details.""")
     if not '.' in hostname:
         sys.exit('Cannot qualify my own hostname, "{}".\nFix /etc/hosts, see man 5 hosts, or add the host to DNS.'.format(hostname))
 
-    # Compute defaults
-    ConfigurationDefaults = Configuration()
-    ServerDefaults = Server()
-    UserDefaults = User()
-
-    # Read the existing configuration.
+   # Read the existing configuration.
     cmd = ['fetchmail', '--configdump', '--nosyslog']
     if rcfile:
         cmd += ['-f', rcfile]
@@ -2235,6 +2256,11 @@ COPYING in the source or documentation directory for details.""")
     except Exception as e:
         sys.exit("Can't parse output of fetchmail --configdump:\n" + str(e))
 
+    # Compute defaults
+    ConfigurationDefaults = Configuration()
+    ServerDefaults = Server()
+    UserDefaults = User()
+
     # The tricky part -- initializing objects from the configuration global
     # `Configuration' is the top level of the object tree we're going to mung.
     # The dictmembers list is used to track the set of fields the dictionary
@@ -2267,6 +2293,10 @@ COPYING in the source or documentation directory for details.""")
     if not rcfile and not dump:
         rcfile = os.environ["HOME"] + "/.fetchmailrc"
 
+    if not have_tk:
+        print("Python's tk (tkinter) module is missing. Please install it to use fetchmailconf.", file=sys.stderr)
+        sys.exit(1)
+
     # OK, now run the configuration edit
     r = Tk()
 
